#!/usr/bin/env python
import os
import sys


CREDFILE = os.path.expanduser('~/.aws/credentials')


with open(CREDFILE, 'r') as f:
    text = f.read()

creds = {}
for group in text.split('['):
    if not group.strip():
        continue
    name, rest = group.strip().split(']')
    creds[name.strip()] = {
        k.strip(): v.strip()
        for k, v in (
            line.split('=')
            for line in rest.strip().split('\n')
            if line and '=' in line
        )
    }

if len(sys.argv) != 2:
    print('Available profiles:\n%s' % ('\n'.join('* %s' % (profile) for profile in creds.keys() if profile != 'default')))
    sys.exit()
profile = sys.argv[1]
wanted = creds[profile]

creds['default'] = wanted

text = '\n\n'.join(
    '[%s]\n%s' % (name, '\n'.join('%s=%s' % (k, v) for k, v in values.items()))
    for name, values in creds.items()
)
with open(CREDFILE, 'w') as f:
    f.write(text)

print('Made %s default' % (profile,))



#!/usr/bin/env python
"""Usage:
    watch_resource.py [--profile=<p>] <arn>...
"""
import collections
import time

import boto3
import docopt


class Error(Exception):
    pass


Descriptor = collections.namedtuple(
    'Descriptor',
    ('service', 'resourcetype', 'desc_func', 'respkey', 'statuskey', 'status_func')
)


Arn = collections.namedtuple(
    'Arn',
    ('arn', 'partition', 'service', 'region', 'account_id', 'resourcetype', 'resource')
)


DESCRIPTORS = [
    Descriptor(
        'rds',
        'db',
        lambda arn: boto3.client('rds').describe_db_instances(DBInstanceIdentifier=arn),
        'DBInstances',
        'DBInstanceStatus',
        lambda arn: boto3.client('rds').describe_db_instances(DBInstanceIdentifier=arn)['DBInstances'][0]['DBInstanceStatus'],
    ),
    Descriptor(
        'ec2',
        'volume',
        lambda arn: boto3.client('ec2').describe_volumes(VolumeIds=[arn.split(':')[-1].split('/')[-1]]),
        'Volumes',
        'State',
        lambda arn: boto3.client('ec2').describe_volumes(VolumeIds=[arn.split(':')[-1].split('/')[-1]])['Volumes'][0]['State'],
    ),
]


ARNMAP = {}


for _ in DESCRIPTORS:
    ARNMAP.setdefault(_.service, {})[_.resourcetype] = _


def main():
    args = docopt.docopt(__doc__)
    if args['--profile']:
        boto3.setup_default_session(profile_name=args['--profile'])
    arn_strs = args['<arn>']
    descriptors = []
    for arn_str in arn_strs:
        arn_parts = arn_str.split(':')
        # Some ARNs (such as for EBS volumes) have 5 colon-separated pieces with the resource type separated from the
        # id with a slash in the last piece.
        if len(arn_parts) == 6:
            (arn_parts[-1], part) = arn_parts[-1].split('/')
            arn_parts.append(part)
        # Other ARNs have 6 colons.
        if len(arn_parts) != 7:
            raise Error('ARN %s does not have the right number of pieces' % (arn_str,))
        arn = Arn(*arn_parts)
        descriptors.append(ARNMAP[arn.service][arn.resourcetype])
    try:
        while True:
            for descriptor in descriptors:
                status = descriptor.status_func(arn_str)
                print(arn_str, status)
            time.sleep(5)
    except KeyboardInterrupt:
        pass


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""Usage:
    tail_stack_events.py [--depth=<d>] [--max-column-length=<x>] [--profile=<profile>] <stack>

Options:
    -p <p> --profile=<profile>      The aws profile to use.
    -d <d> --depth=<d>              The maximum depth to get events for. Use -1 for unlimited depth. [default: 2]
    -x <x> --max-col-length=<x>     The maximum columns length for tabular output.
                                    Defaults to 200 for postmortem, 40 otherwise.
    <stack>                         The top-level stack to get events for.
"""
import collections
import logging
import math
import sys

import eventlet
eventlet.monkey_patch()

import ansiwrap
import botocore.exceptions
import boto3
import colorama
import docopt
import eventlet.greenpool
import tenacity


STACK_TYPE = 'AWS::CloudFormation::Stack'
ELLIPSIS = u'\u2026'

LOG = logging.getLogger(__name__)


def retry(func):
    tretry = tenacity.retry(
        wait=(
            tenacity.wait_random_exponential(multiplier=1, min=0.1, max=10)
        ),
        after=tenacity.after_log(LOG, logging.WARNING),
    )
    def log_exc(*a, **k):
        try:
            return func(*a, **k)
        except Exception:
            LOG.exception('Exception calling %r' % (func,))
            raise
    return tretry(log_exc)


class Column(object):
    def __init__(self, mvl, ml):
        self.max_value_length = mvl
        self.max_length = ml


@retry
def get_stack(stack_name_or_arn):
    cf = boto3.resource('cloudformation')
    stack = cf.Stack(stack_name_or_arn)
    # Switch to the ARN if a stack name was passed in
    if stack.stack_id != stack_name_or_arn:
        stack = cf.Stack(stack.stack_id)
    return stack


def update_columns(columns, events):
    for event in events:
        for column in columns.keys():
            columns[column].max_value_length = max(columns[column].max_value_length, len(str(getattr(event, column))))


def format_column(column_name, column, value):
    text = str(value)
    if column_name == 'resource_status':
        uvalue = value.upper()
        if 'FAIL' in uvalue:
            color = colorama.Fore.RED
        elif 'ROLLBACK' in uvalue:
            color = colorama.Fore.YELLOW
        elif 'IN_PROGRESS' in uvalue:
            color = colorama.Fore.BLUE
        elif uvalue == 'DELETE_COMPLETE':
            color = colorama.Fore.LIGHTBLACK_EX
        elif 'COMPLETE' in uvalue:
            color = colorama.Fore.GREEN
        else:
            color = colorama.Fore.WHITE
        text = '%s%s%s' % (color, value, colorama.Style.RESET_ALL)
    if ansiwrap.ansilen(text) > column.max_length:
        half_length = (column.max_length - 1) / 2.0
        output_text = '%s%s%s' % (
            text[:int(math.floor(half_length))],
            ELLIPSIS,
            text[-int(math.ceil(half_length)):],
        )
    else:
        output_text = text
    padding = ' ' * max(0, min(column.max_value_length, column.max_length) - ansiwrap.ansilen(output_text))
    return '%s%s' % (
        padding,
        output_text,
    )


def output_events(columns, events):
    for e in events:
        fmt = '  '.join('%s' for _ in columns.values())
        print(fmt % tuple([
            format_column(n, c, getattr(e, n)) for n, c in columns.items()
        ]))


@retry
def next_page(pages):
    try:
        return list(next(pages))
    except botocore.exceptions.ClientError:
        return None
        # traceback.print_exc()
    except StopIteration:
        return None


def short_stack_name(name):
    if ':' not in name:
        return name
    name = name.split(':')[-1]
    if '/' not in name:
        return name
    return name.split('/')[1]


def get_pending_resources(stack):
    pending_resources = []
    for sub in stack.resource_summaries.all():
        if (
            ('IN_PROGRESS' not in sub.resource_status and 'FAILED' not in sub.resource_status)
            or 'COMPLETE' in sub.resource_status
        ):
            continue
        sub.short_stack_name = short_stack_name(sub.stack_name)
        pending_resources.append(sub)
        if sub.resource_type == STACK_TYPE:
            pending_resources.extend(get_pending_resources(get_stack(sub.physical_resource_id)))
    return pending_resources


def main():
    args = docopt.docopt(__doc__)
    if args['--profile']:
        boto3.setup_default_session(profile_name=args['--profile'])

    max_column_length = args['--max-column-length']
    if max_column_length is None:
        max_column_length = 200

    columns = collections.OrderedDict([
        ('short_stack_name', Column(0, max_column_length)),
        ('logical_resource_id', Column(0, max_column_length)),
        # ('stack_id', Column(0, max_column_length)),
        ('resource_type', Column(0, max_column_length)),
        ('resource_status', Column(0, max_column_length)),
        ('resource_status_reason', Column(0, max_column_length)),
    ])
    headers = collections.namedtuple('Headers', columns.keys())(*[
        colorama.Style.BRIGHT + t + colorama.Style.RESET_ALL
        for t in [
            'Stack Name',
            'Logical Resource ID',
            # 'Stack ID',
            'Resource Type',
            'Status',
            'Reason',
        ]
    ])
    update_columns(columns, [headers])

    print('Getting stack...')
    main_stack = get_stack(args['<stack>'])

    pending_resources = get_pending_resources(main_stack)
    if not pending_resources:
        print('None')
        sys.exit(0)

    update_columns(columns, pending_resources)

    output_events(columns, [headers])
    output_events(columns, pending_resources)


if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        sys.exit(0)



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['aws_utilities']

package_data = \
{'': ['*']}

install_requires = \
['ansiwrap>=0.8.4,<0.9.0',
 'boto3>=1.9,<2.0',
 'colorama>=0.4.1,<0.5.0',
 'docopt>=0.6.2,<0.7.0',
 'ec2-metadata>=2.0,<3.0',
 'eventlet>=0.24.1,<0.25.0',
 'tenacity>=5.0,<6.0',
 'urllib3>=1.24,<1.25']

entry_points = \
{'console_scripts': ['pending_stack_resources = '
                     'aws_utilities.pending_stack_resources:main',
                     'tail_cloudwatch_logs = '
                     'aws_utilities.tail_cloudwatch_logs:main',
                     'tail_stack_events = aws_utilities.tail_stack_events:main',
                     'wait_for_stack_complete = '
                     'aws_utilities.wait_for_stack_complete:main',
                     'watch_resource = aws_utilities.watch_resource:main']}

setup_kwargs = {
    'name': 'aws-utilities',
    'version': '1.2.0',
    'description': '',
    'long_description': "# aws-utilities\n\nThis package includes various command-line utilities for use with aws.\n\nTo set up a local checkout with [pyenv](https://github.com/pyenv/pyenv) run these commands:\n```\ngit clone https://github.com/reversefold/aws-utilities.git\ncd aws-utilities\npyenv virtualenv 3.6.5 aws-utilities\npyenv local aws-utilities\npip install -r dev-requirements.txt\n./sync-requirements.sh\n```\n\n\n## tail_cloudwatch_logs.py\n\nGet the last `n` lines of a cloudwatch log group and follow the output in realtime as it is written to CloudWatch Logs. Has the ability to use any profile set up in your `~/.aws/credentials` so working across multiple accounts is easy.\n\nInspired by [cw](https://github.com/lucagrulla/cw).\n\n\n## tail_stack_events.py\n\nGet the last `n` events for a CloudFormation stack and all of its nested stacks and follow the events in realtime. This utility can give you a view into all of the events happening in any size CloudFormation stack, even if it has multiple levels of nested stacks. When this script is started up it finds all nested stacks and follows their events as well if the stack is in any status which includes IN_PROGRESS. When following stack events, nested stacks will be dynamically added to and removed from the set of stacks being queried for events as nested stacks go into the various `IN_PROGRESS` and `COMPLETE` states. This lets you get a complete picture of what is going on while also making the minimum number of API calls.\n\nIn postmortem mode this script will find the events that caused the last stack update to fail. It will follow nested stack failures until it finds the specific resource that caused the failure.\n\nOriginally inspired by [tail-stack-events](https://github.com/tmont/tail-stack-events) and [cfn-tail](https://github.com/taimos/cfn-tail).\n\n\n## aws_switch.py\n\nA quick and dirty script to make any one of your configured aws profiles the default profile. Useful when you're using tools which don't support profiles or when you work in distinct profiles at distinct times.\n\n\n## wait_for_stack_complete.py\n\nA simple script for running on an ec2 instance. No parameters are taken. Finds the CloudFormation stack that the instance resides in and polls until the stack is in a `COMPLETE` state. If the stack has a parent stack it will watch that one instead. Has retries with exponential backoff (up to 5m) for all API calls so as to not overload the AWS APIs when used in a large environment. This script is particularly useful for UserData or cfn-init scripts which need to wait for other resources to be created and attached, such as EBS volumes not included in the instance's BlockDeviceMapping.\n",
    'author': 'Justin Patrin',
    'author_email': 'papercrane@reversefold.com',
    'url': 'https://github.com/reversefold/aws-utilities',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
